@@ -1,16 +1,53 @@
 use strict;
 use warnings;
 package Version::Requirements;
-BEGIN {
-  $Version::Requirements::VERSION = '0.101010';
-}
 # ABSTRACT: a set of version requirements for a CPAN dist
-
+$Version::Requirements::VERSION = '0.101023';
+#pod =head1 SYNOPSIS
+#pod
+#pod   use Version::Requirements;
+#pod
+#pod   my $build_requires = Version::Requirements->new;
+#pod
+#pod   $build_requires->add_minimum('Library::Foo' => 1.208);
+#pod
+#pod   $build_requires->add_minimum('Library::Foo' => 2.602);
+#pod
+#pod   $build_requires->add_minimum('Module::Bar'  => 'v1.2.3');
+#pod
+#pod   $METAyml->{build_requires} = $build_requires->as_string_hash;
+#pod
+#pod =head1 DESCRIPTION
+#pod
+#pod A Version::Requirements object models a set of version constraints like those
+#pod specified in the F<META.yml> or F<META.json> files in CPAN distributions.  It
+#pod can be built up by adding more and more constraints, and it will reduce them to
+#pod the simplest representation.
+#pod
+#pod Logically impossible constraints will be identified immediately by thrown
+#pod exceptions.
+#pod
+#pod =cut
 
 use Carp ();
 use Scalar::Util ();
 use version 0.77 (); # the ->parse method
 
+# We silence this warning during core tests, because this is only in core
+# because it has to be, and nobody wants to see this stupid warning.
+# -- rjbs, 2012-01-20
+Carp::cluck(
+  "Version::Requirements is deprecated; replace with CPAN::Meta::Requirements"
+) unless $ENV{PERL_CORE};
+
+#pod =method new
+#pod
+#pod   my $req = Version::Requirements->new;
+#pod
+#pod This returns a new Version::Requirements object.  It ignores any arguments
+#pod given.
+#pod
+#pod =cut
 
 sub new {
   my ($class) = @_;
@@ -27,6 +64,57 @@ sub _version_object {
   return $version;
 }
 
+#pod =method add_minimum
+#pod
+#pod   $req->add_minimum( $module => $version );
+#pod
+#pod This adds a new minimum version requirement.  If the new requirement is
+#pod redundant to the existing specification, this has no effect.
+#pod
+#pod Minimum requirements are inclusive.  C<$version> is required, along with any
+#pod greater version number.
+#pod
+#pod This method returns the requirements object.
+#pod
+#pod =method add_maximum
+#pod
+#pod   $req->add_maximum( $module => $version );
+#pod
+#pod This adds a new maximum version requirement.  If the new requirement is
+#pod redundant to the existing specification, this has no effect.
+#pod
+#pod Maximum requirements are inclusive.  No version strictly greater than the given
+#pod version is allowed.
+#pod
+#pod This method returns the requirements object.
+#pod
+#pod =method add_exclusion
+#pod
+#pod   $req->add_exclusion( $module => $version );
+#pod
+#pod This adds a new excluded version.  For example, you might use these three
+#pod method calls:
+#pod
+#pod   $req->add_minimum( $module => '1.00' );
+#pod   $req->add_maximum( $module => '1.82' );
+#pod
+#pod   $req->add_exclusion( $module => '1.75' );
+#pod
+#pod Any version between 1.00 and 1.82 inclusive would be acceptable, except for
+#pod 1.75.
+#pod
+#pod This method returns the requirements object.
+#pod
+#pod =method exact_version
+#pod
+#pod   $req->exact_version( $module => $version );
+#pod
+#pod This sets the version required for the given module to I<exactly> the given
+#pod version.  No other version would be considered acceptable.
+#pod
+#pod This method returns the requirements object.
+#pod
+#pod =cut
 
 BEGIN {
   for my $type (qw(minimum maximum exclusion exact_version)) {
@@ -38,9 +126,7 @@ BEGIN {
 
       $version = $self->_version_object( $version );
 
-      my $old = $self->{ $name } || 'Version::Requirements::_Range::Range';
-
-      $self->{ $name } = $old->$method($version);
+      $self->__modify_entry_for($name, $method, $version);
 
       return $self;
     };
@@ -50,6 +136,17 @@ BEGIN {
   }
 }
 
+#pod =method add_requirements
+#pod
+#pod   $req->add_requirements( $another_req_object );
+#pod
+#pod This method adds all the requirements in the given Version::Requirements object
+#pod to the requirements object on which it was called.  If there are any conflicts,
+#pod an exception is thrown.
+#pod
+#pod This method returns the requirements object.
+#pod
+#pod =cut
 
 sub add_requirements {
   my ($self, $req) = @_;
@@ -65,6 +162,22 @@ sub add_requirements {
   return $self;
 }
 
+#pod =method accepts_module
+#pod
+#pod   my $bool = $req->accepts_modules($module => $version);
+#pod
+#pod Given an module and version, this method returns true if the version
+#pod specification for the module accepts the provided version.  In other words,
+#pod given:
+#pod
+#pod   Module => '>= 1.00, < 2.00'
+#pod
+#pod We will accept 1.00 and 1.75 but not 0.50 or 2.00.
+#pod
+#pod For modules that do not appear in the requirements, this method will return
+#pod true.
+#pod
+#pod =cut
 
 sub accepts_module {
   my ($self, $module, $version) = @_;
@@ -75,17 +188,46 @@ sub accepts_module {
   return $range->_accepts($version);
 }
 
+#pod =method clear_requirement
+#pod
+#pod   $req->clear_requirement( $module );
+#pod
+#pod This removes the requirement for a given module from the object.
+#pod
+#pod This method returns the requirements object.
+#pod
+#pod =cut
 
 sub clear_requirement {
   my ($self, $module) = @_;
-  delete $self->{ $module };
+
+  return $self unless $self->__entry_for($module);
+
+  Carp::confess("can't clear requirements on finalized requirements")
+    if $self->is_finalized;
+
+  delete $self->{requirements}{ $module };
 
   return $self;
 }
 
+#pod =method required_modules
+#pod
+#pod This method returns a list of all the modules for which requirements have been
+#pod specified.
+#pod
+#pod =cut
 
-sub required_modules { keys %{ $_[ 0 ] } }
+sub required_modules { keys %{ $_[0]{requirements} } }
 
+#pod =method clone
+#pod
+#pod   $req->clone;
+#pod
+#pod This method returns a clone of the invocant.  The clone and the original object
+#pod can then be changed independent of one another.
+#pod
+#pod =cut
 
 sub clone {
   my ($self) = @_;
@@ -94,10 +236,32 @@ sub clone {
   return $new->add_requirements($self);
 }
 
-sub __entry_for {
-  $_[0]{ $_[1] }
+sub __entry_for     { $_[0]{requirements}{ $_[1] } }
+
+sub __modify_entry_for {
+  my ($self, $name, $method, $version) = @_;
+
+  my $fin = $self->is_finalized;
+  my $old = $self->__entry_for($name);
+
+  Carp::confess("can't add new requirements to finalized requirements")
+    if $fin and not $old;
+
+  my $new = ($old || 'Version::Requirements::_Range::Range')
+          ->$method($version);
+
+  Carp::confess("can't modify finalized requirements")
+    if $fin and $old->as_string ne $new->as_string;
+
+  $self->{requirements}{ $name } = $new;
 }
 
+#pod =method is_simple
+#pod
+#pod This method returns true if and only if all requirements are inclusive minimums
+#pod -- that is, if their string expression is just the version number.
+#pod
+#pod =cut
 
 sub is_simple {
   my ($self) = @_;
@@ -109,15 +273,80 @@ sub is_simple {
   return 1;
 }
 
+#pod =method is_finalized
+#pod
+#pod This method returns true if the requirements have been finalized by having the
+#pod C<finalize> method called on them.
+#pod
+#pod =cut
+
+sub is_finalized { $_[0]{finalized} }
+
+#pod =method finalize
+#pod
+#pod This method marks the requirements finalized.  Subsequent attempts to change
+#pod the requirements will be fatal, I<if> they would result in a change.  If they
+#pod would not alter the requirements, they have no effect.
+#pod
+#pod If a finalized set of requirements is cloned, the cloned requirements are not
+#pod also finalized.
+#pod
+#pod =cut
+
+sub finalize { $_[0]{finalized} = 1 }
+
+#pod =method as_string_hash
+#pod
+#pod This returns a reference to a hash describing the requirements using the
+#pod strings in the F<META.yml> specification.
+#pod
+#pod For example after the following program:
+#pod
+#pod   my $req = Version::Requirements->new;
+#pod
+#pod   $req->add_minimum('Version::Requirements' => 0.102);
+#pod
+#pod   $req->add_minimum('Library::Foo' => 1.208);
+#pod
+#pod   $req->add_maximum('Library::Foo' => 2.602);
+#pod
+#pod   $req->add_minimum('Module::Bar'  => 'v1.2.3');
+#pod
+#pod   $req->add_exclusion('Module::Bar'  => 'v1.2.8');
+#pod
+#pod   $req->exact_version('Xyzzy'  => '6.01');
+#pod
+#pod   my $hashref = $req->as_string_hash;
+#pod
+#pod C<$hashref> would contain:
+#pod
+#pod   {
+#pod     'Version::Requirements' => '0.102',
+#pod     'Library::Foo' => '>= 1.208, <= 2.206',
+#pod     'Module::Bar'  => '>= v1.2.3, != v1.2.8',
+#pod     'Xyzzy'        => '== 6.01',
+#pod   }
+#pod
+#pod =cut
 
 sub as_string_hash {
   my ($self) = @_;
 
-  my %hash = map {; $_ => $self->{$_}->as_string } keys %$self;
+  my %hash = map {; $_ => $self->{requirements}{$_}->as_string }
+             $self->required_modules;
 
   return \%hash;
 }
 
+#pod =method from_string_hash
+#pod
+#pod   my $req = Version::Requirements->from_string_hash( \%hash );
+#pod
+#pod This is an alternate constructor for a Version::Requirements object.  It takes
+#pod a hash of module names and version requirement strings and returns a new
+#pod Version::Requirements object.
+#pod
+#pod =cut
 
 my %methods_for_op = (
   '==' => [ qw(exact_version) ],
@@ -157,9 +386,6 @@ sub from_string_hash {
 {
   package
     Version::Requirements::_Range::Exact;
-BEGIN {
-  $Version::Requirements::_Range::Exact::VERSION = '0.101010';
-}
   sub _new     { bless { version => $_[1] } => $_[0] }
 
   sub _accepts { return $_[0]{version} == $_[1] }
@@ -168,29 +394,33 @@ BEGIN {
 
   sub as_modifiers { return [ [ exact_version => $_[0]{version} ] ] }
 
+  sub _clone {
+    (ref $_[0])->_new( version->new( $_[0]{version} ) )
+  }
+
   sub with_exact_version {
     my ($self, $version) = @_;
 
-    return $self if $self->_accepts($version);
+    return $self->_clone if $self->_accepts($version);
 
     Carp::confess("illegal requirements: unequal exact version specified");
   }
 
   sub with_minimum {
     my ($self, $minimum) = @_;
-    return $self if $self->{version} >= $minimum;
+    return $self->_clone if $self->{version} >= $minimum;
     Carp::confess("illegal requirements: minimum above exact specification");
   }
 
   sub with_maximum {
     my ($self, $maximum) = @_;
-    return $self if $self->{version} <= $maximum;
+    return $self->_clone if $self->{version} <= $maximum;
     Carp::confess("illegal requirements: maximum below exact specification");
   }
 
   sub with_exclusion {
     my ($self, $exclusion) = @_;
-    return $self unless $exclusion == $self->{version};
+    return $self->_clone unless $exclusion == $self->{version};
     Carp::confess("illegal requirements: excluded exact specification");
   }
 }
@@ -200,12 +430,25 @@ BEGIN {
 {
   package
     Version::Requirements::_Range::Range;
-BEGIN {
-  $Version::Requirements::_Range::Range::VERSION = '0.101010';
-}
 
   sub _self { ref($_[0]) ? $_[0] : (bless { } => $_[0]) }
 
+  sub _clone {
+    return (bless { } => $_[0]) unless ref $_[0];
+
+    my ($s) = @_;
+    my %guts = (
+      (exists $s->{minimum} ? (minimum => version->new($s->{minimum})) : ()),
+      (exists $s->{maximum} ? (maximum => version->new($s->{maximum})) : ()),
+
+      (exists $s->{exclusions}
+        ? (exclusions => [ map { version->new($_) } @{ $s->{exclusions} } ])
+        : ()),
+    );
+
+    bless \%guts => ref($s);
+  }
+
   sub as_modifiers {
     my ($self) = @_;
     my @mods;
@@ -249,7 +492,7 @@ BEGIN {
 
   sub with_exact_version {
     my ($self, $version) = @_;
-    $self = $self->_self;
+    $self = $self->_clone;
 
     Carp::confess("illegal requirements: exact specification outside of range")
       unless $self->_accepts($version);
@@ -289,7 +532,7 @@ BEGIN {
 
   sub with_minimum {
     my ($self, $minimum) = @_;
-    $self = $self->_self;
+    $self = $self->_clone;
 
     if (defined (my $old_min = $self->{minimum})) {
       $self->{minimum} = (sort { $b cmp $a } ($minimum, $old_min))[0];
@@ -302,7 +545,7 @@ BEGIN {
 
   sub with_maximum {
     my ($self, $maximum) = @_;
-    $self = $self->_self;
+    $self = $self->_clone;
 
     if (defined (my $old_max = $self->{maximum})) {
       $self->{maximum} = (sort { $a cmp $b } ($maximum, $old_max))[0];
@@ -315,7 +558,7 @@ BEGIN {
 
   sub with_exclusion {
     my ($self, $exclusion) = @_;
-    $self = $self->_self;
+    $self = $self->_clone;
 
     push @{ $self->{exclusions} ||= [] }, $exclusion;
 
@@ -337,15 +580,18 @@ BEGIN {
 1;
 
 __END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Version::Requirements - a set of version requirements for a CPAN dist
 
 =head1 VERSION
 
-version 0.101010
+version 0.101023
 
 =head1 SYNOPSIS
 
@@ -480,6 +726,20 @@ can then be changed independent of one another.
 This method returns true if and only if all requirements are inclusive minimums
 -- that is, if their string expression is just the version number.
 
+=head2 is_finalized
+
+This method returns true if the requirements have been finalized by having the
+C<finalize> method called on them.
+
+=head2 finalize
+
+This method marks the requirements finalized.  Subsequent attempts to change
+the requirements will be fatal, I<if> they would result in a change.  If they
+would not alter the requirements, they have no effect.
+
+If a finalized set of requirements is cloned, the cloned requirements are not
+also finalized.
+
 =head2 as_string_hash
 
 This returns a reference to a hash describing the requirements using the
@@ -522,7 +782,13 @@ Version::Requirements object.
 
 =head1 AUTHOR
 
-  Ricardo Signes <rjbs@cpan.org>
+Ricardo Signes <rjbs@cpan.org>
+
+=head1 CONTRIBUTOR
+
+=for stopwords Karen Etheridge
+
+Karen Etheridge <ether@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
@@ -532,4 +798,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
